// 有序单向链表的插入（链表有序性的谓词描述）

#include <stdlib.h>    // malloc()函数、NULL宏

typedef struct node {
    struct node * next;
    int data; 
} Node;
//@ shape next : list; // 形状标注：指针域next构成单向链表

//@ logic Node *oldhead; // 记录函数入口处链表头节点（实参）的值
//@ logic int m;         // 用于表示函数入口处链表的长度。插入操作结束后链表的长度为m+1

/*@
inductive sorted_list(Node* p) = // 有序单向链表的定义：分成三种情况，从右向左进行归纳
    p == \null ||				 // 空表
    p != \null && p->next == \null ||  // 单个节点
    p != \null && p->next != \null &&  // 2个节点以上
	p->data <= p->next->data && sorted_list(p->next);
inductive sorted_seg(Node *p, Node *q) = // 有序单向链表表段的定义：分成两种情况，从右向左进行归纳
    p == q && p != \null ||                       // 单个节点
    p != q && p != \null && p->next != \null &&   // 2个节点以上
	p->data <= p->next->data && sorted_seg(p->next, q);
*/
/*@ lemma property1:  // 引理1：左边表段 + 右边节点, 构成一个增加了一个节点的新表段，
    \forall Node *p, Node *q. 
        sorted_seg(p, q) && q->data <= q->next->data && q->next != \null
    ==> sorted_seg(p, q->next);
*/
/*@ lemma property2:  // 引理2：左边表段 + 右边链表，构成一个更长的链表
    \forall Node *p, Node *q.
        sorted_seg(p, q) && sorted_list(q)
    ==> sorted_list(p);
*/

/*@ requires 
        sorted_list(head) && \length(head, next) == m && m >= 0 
        && oldhead == head;
    assigns *oldhead;
    exits \exit_status == 1;
    ensures 
        sorted_list(\result) && \length(\result, next) == m+1 // 插在表中或表尾
        && oldhead == \result && m > 0 
        ||
        sorted_list(\result) && \length(\result, next) == m+1 // 生成仅一个元素的表
        && oldhead == \null && m == 0 
        || 
        sorted_list(\result) && \length(\result, next) == m+1 // 插在表头
        && oldhead == \result->(next:1) && m > 0; 
*/     
Node* listInsert(Node* head, const int data) { 
    Node *ptr, *ptr1, *p; 
    //@ ghost int j; 

    p = (Node*)malloc(sizeof(Node));
    if (p == NULL) {exit(1);} 

    p->data = data; 
    p->next = NULL;
    if (head == NULL) {                // 空表
        head = p;                  
    }else if (p->data <= head->data) { 
        p->next = head; head = p;      // 插在表头
    }else {                        
        ptr1 = head;                   // 插在表中或表尾
        ptr = head->next; 
        //@ ghost j = 0; 

        /*@ loop invariant	// 分成插在表中和表尾两种情况
            (ptr != \null && ptr == ptr1->next && ptr1->data <= ptr->data // 循环中（包括新节点插在表中的情况）
              && sorted_list(ptr) && \length(ptr, next) == m-j-1 
            ||
             ptr == \null && ptr1->next == \null  // 循环出口，新节点插在表尾
              && sorted_list(ptr1) &&\length(ptr1, next) == m-j
            ) 
            && oldhead == head && sorted_seg(head, ptr1) 
            && ptr1 == head->(next:j) && ptr1->data < p->data && \list(p) 
            && \length(p, next) == 1 && 0 < m && 0 <= j < m; 
        */
	    while ((ptr != NULL) && (ptr->data < p->data)) {
	        ptr1 = ptr; 
            ptr = ptr->next; 
            //@ ghost j = j + 1;
	    }
	    p->next = ptr1->next;  
        ptr1->next = p;
    }
    return head; 
}